@@ -36,6 +36,7 @@
 #define UDM_DB_MIMER			15
 #define UDM_DB_CACHE			16
 #define UDM_DB_SYBASE			17
+#define UDM_DB_SQLITE3			18
 #define UDM_DB_SEARCHD			200
 
 #define UDM_URL_ACTION_DELETE           0
@@ -95,6 +96,7 @@
 
 extern void	UdmDBFree(void *db);
 
+extern int UdmURLActionNoLock(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd);
 extern __C_LINK int __UDMCALL UdmURLAction(UDM_AGENT *A, UDM_DOCUMENT   *D, int cmd);
 extern __C_LINK int __UDMCALL UdmResAction(UDM_AGENT *A, UDM_RESULT     *R, int cmd);
 extern __C_LINK int __UDMCALL UdmCatAction(UDM_AGENT *A, UDM_CATEGORY   *C, int cmd);
@@ -151,7 +153,8 @@ extern int UdmResAddDocInfoSQL(UDM_AGENT *query, UDM_DB *db, UDM_RESULT *Res, si
 extern int UdmTrackSQL(UDM_AGENT * query, UDM_RESULT *Res, UDM_DB *db);
 extern int UdmCloneListSQL(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc, UDM_RESULT *Res, UDM_DB *db);
 extern int UdmTargetsSQL(UDM_AGENT *Indexer, UDM_DB *db);
-extern int UdmUserCacheStoreSQL(UDM_AGENT *Indexer, UDM_RESULT *Res, UDM_DB *db);
+extern int UdmQueryCachePutSQL(UDM_AGENT *Indexer, UDM_RESULT *Res, UDM_DB *db);
+extern int UdmQueryCacheGetSQL(UDM_AGENT *Indexer, UDM_RESULT *Res, UDM_DB *db);
 #endif
 
 /* MultiCache stuff. */
