@@ -163,6 +163,7 @@ extern UDM_SQLDB_HANDLER udm_sqldb_mysql_handler;
 extern UDM_SQLDB_HANDLER udm_sqldb_pgsql_handler;
 extern UDM_SQLDB_HANDLER udm_sqldb_ibase_handler;
 extern UDM_SQLDB_HANDLER udm_sqldb_sqlite_handler;
+extern UDM_SQLDB_HANDLER udm_sqldb_sqlite3_handler;
 extern UDM_SQLDB_HANDLER udm_sqldb_odbc_handler;
 extern UDM_SQLDB_HANDLER udm_sqldb_oracle_handler;
 
@@ -175,7 +176,7 @@ typedef struct udm_db_st
         char    *from;
 	int	DBType;
 	int	DBDriver;
-	
+	int	version; /* vendor specific version */
 	int	DBSQL_IN;
         int     flags;
 	
