@@ -46,10 +46,14 @@
 #endif
 #endif
 
-typedef struct {
-  int handle;
-  int cnt;
+typedef struct
+{
   udm_mutex_t mutex;
+#ifdef HAVE_DEBUG
+  int handle;
+  int count;
+  pthread_t thread;
+#endif
 } UDM_MUTEX;
 
 
@@ -66,12 +70,13 @@ extern __C_LINK void __UDMCALL UdmDestroyMutexes(void);
 extern __C_LINK udm_mutex_t ThreadProcLock;
 #endif
 
-#define UDM_GETLOCK(A,mutex)		if(A->Conf->LockProc)A->Conf->LockProc(A,UDM_LOCK,mutex,__FILE__,__LINE__)
-#define UDM_RELEASELOCK(A,mutex)	if(A->Conf->LockProc)A->Conf->LockProc(A,UDM_UNLOCK,mutex,__FILE__,__LINE__)
+#define UDM_GETLOCK(A,mutex)		if((A)->Conf->LockProc)(A)->Conf->LockProc((A),UDM_LOCK,(mutex),__FILE__,__LINE__)
+#define UDM_RELEASELOCK(A,mutex)	if((A)->Conf->LockProc)(A)->Conf->LockProc((A),UDM_UNLOCK,(mutex),__FILE__,__LINE__)
+#define UDM_LOCK_CHECK_OWNER(A,mutex)	if((A)->Conf->LockProc)(A)->Conf->LockProc((A),UDM_CKLOCK,(mutex),__FILE__,__LINE__)
 
 /* Locking commands */
 #define UDM_LOCK		1
 #define UDM_UNLOCK		2
-
+#define UDM_CKLOCK		3
 
 #endif
