@@ -21,8 +21,9 @@
 #define UDM_CHARSET_H
 
 #define UDM_RECODE_TEXT                 0
-#define UDM_RECODE_HTML                 1
-
+#define UDM_RECODE_HTML_NONASCII	1
+#define UDM_RECODE_HTML_SPECIAL		2
+#define UDM_RECODE_HTML                 3
 
 #define UDM_CHARSET_UNKNOWN             0
 #define UDM_CHARSET_ARABIC              1
