@@ -50,6 +50,8 @@ extern __C_LINK int __UDMCALL UdmVarListReplaceInt(UDM_VARLIST * vars,const char
 extern __C_LINK int __UDMCALL UdmVarListReplaceUnsigned(UDM_VARLIST * vars, const char * name, uint4 val);
 extern int UdmVarListReplaceDouble(UDM_VARLIST * vars, const char * name, double val);
 
+size_t UdmDSTRParse(UDM_DSTR *d, const char *fmt, UDM_VARLIST *L);
+
 extern int UdmVarListInsStr(UDM_VARLIST * vars,const char * name,const char * val);
 extern int UdmVarListInsInt(UDM_VARLIST * vars,const char * name,int val);
 
@@ -62,5 +64,5 @@ extern __C_LINK int          UdmVarListFindBool(UDM_VARLIST * vars,const char *
 extern __C_LINK unsigned     UdmVarListFindUnsigned(UDM_VARLIST * vars, const char * name, unsigned defval);
 extern __C_LINK double       UdmVarListFindDouble(UDM_VARLIST * vars, const char * name, double defval);
 
-
+extern int UdmVarListConvert(UDM_VARLIST *L, UDM_CONV *conv);
 #endif
