@@ -1,6 +1,6 @@
 #!/usr/bin/perl
-# $Source: /usr/home/bar/cvsroot/mnogosearch32/install.pl,v $
-# $Id: install.pl,v 1.6 2004/12/23 12:21:08 svoj Exp $
+# $Source: /home/bar/cvsroot/mnogosearch32/install.pl,v $
+# $Id: install.pl,v 1.7 2006/02/25 16:50:49 bar Exp $
 
 use Cwd 'abs_path';
 
@@ -17,6 +17,7 @@ $build_static='yes';
 $syslog='yes';
 $syslog_facility='default';
 
+$docs='no';
 $pthreads='yes';
 $parser='yes';
 $mp3='yes';
@@ -91,6 +92,12 @@ sub configure {
         	}
         }
 
+        if ($docs eq 'yes') {
+        	push @arg,"--with-docs";
+        } elsif ($docs eq 'no') {
+        	push @arg,"--without-docs";
+	}
+
         if ($pthreads eq 'yes') {
         	push @arg,"--enable-pthreads";
         } elsif ($pthreads eq 'no') {
@@ -642,6 +649,17 @@ sub set_syslog {
 sub set_other {
 	print "Additional features\n";
 	print "-------------------\n";
+	while () {
+		print "Build documentation (docbook required)? (yes/no) [$docs]: ";
+		$temp=read_in();
+		if (($temp eq '') || ($temp eq $docs)) {
+			last;			
+		} elsif (($temp eq 'yes') || ($temp eq 'no')) {
+			$docs=$temp;
+			last;
+		}
+	}
+	
 	while () {
 		print "Enable Posix pthreads? (yes/no) [$pthreads]: ";
 		$temp=read_in();
