@@ -1290,10 +1290,12 @@ static UDM_CONFCMD commands[] =
   {"AlwaysFoundWord",      1,1,   env_rpl_var},
   {"CustomLog",            1,1,   env_rpl_var},
   {"CurrentTime",          1,1,   env_rpl_var},
+  {"Locale",               1,1,   env_rpl_var},
   {"WordDistanceWeight",   1,1,   env_rpl_num_var},
   {"MaxCoordFactor",       1,1,   env_rpl_num_var},
   {"MinCoordFactor",       1,1,   env_rpl_num_var},
   {"NumWordFactor",        1,1,   env_rpl_var},
+  {"NumDistinctWordFactor",1,1,   env_rpl_num_var},
   {"URLDataThreshold",     1,1,   env_rpl_num_var},
   {"DocSizeWeight",        1,1,   env_rpl_num_var},
   {"RelevancyFactor",      1,1,   env_rpl_num_var},
