@@ -197,6 +197,8 @@ UdmRegisterChild(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
   const char  *parent=NULL;
   int    parent_id=0;
     
+  UDM_LOCK_CHECK_OWNER(Indexer, UDM_LOCK_CONF);
+  
   if((Sec=UdmVarListFind(&Doc->Sections,"Header.References")) && Sec->val)
   {
     /* 
@@ -222,7 +224,7 @@ UdmRegisterChild(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
       
     UdmDocInit(&Msg);
     UdmVarListReplaceStr(&Msg.Sections,"Header.Message-ID",parent);
-    result = UdmURLAction(Indexer, &Msg, UDM_URL_ACTION_FINDBYMSG);
+    result= UdmURLActionNoLock(Indexer, &Msg, UDM_URL_ACTION_FINDBYMSG);
     parent_id = UdmVarListFindInt(&Msg.Sections,"ID",0);
     UdmVarListReplaceInt(&Doc->Sections,"Header.Parent-ID",parent_id);
     UdmDocFree(&Msg);
@@ -230,7 +232,7 @@ UdmRegisterChild(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
   
   /* Now register me with my parent  */
   if(parent_id)
-    result = UdmURLAction(Indexer, Doc, UDM_URL_ACTION_REGCHILD);
+    result = UdmURLActionNoLock(Indexer, Doc, UDM_URL_ACTION_REGCHILD);
   return result;
 }
 
@@ -245,16 +247,16 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
   int        use_newsext; 
   int        action;
 
-  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+  UDM_LOCK_CHECK_OWNER(Indexer, UDM_LOCK_CONF);
+
   use_newsext= !strcasecmp(UdmVarListFindStr(&Indexer->Conf->Vars,"NewsExtensions","no"),"yes");
-  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
   
   /* First of all check that URL must be delated */
   
   if(Doc->method==UDM_METHOD_DISALLOW)
   {
     UdmLog(Indexer,UDM_LOG_ERROR,"Deleting %s", UdmVarListFindStr(&Doc->Sections, "URL", ""));
-    rc= UdmURLAction(Indexer, Doc, UDM_URL_ACTION_DELETE);
+    rc= UdmURLActionNoLock(Indexer, Doc, UDM_URL_ACTION_DELETE);
     return rc;
   }
 
@@ -287,7 +289,7 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
       if(Doc->method == UDM_METHOD_GET && Doc->Spider.use_clones)
       {
         urlid_t    origin_id = 0;
-        if (UDM_OK != (rc= UdmURLAction(Indexer, Doc, UDM_URL_ACTION_FINDORIG)))
+        if (UDM_OK != (rc= UdmURLActionNoLock(Indexer, Doc, UDM_URL_ACTION_FINDORIG)))
           return rc;
         origin_id= (urlid_t)UdmVarListFindInt(&Doc->Sections,"Origin-ID",0);
   
@@ -375,7 +377,7 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
 
   UdmTime_t2HttpStr(next_index_time,dbuf);
   UdmVarListReplaceStr(&Doc->Sections,"Next-Index-Time",dbuf);  
-  rc= UdmURLAction(Indexer, Doc, action);
+  rc= UdmURLActionNoLock(Indexer, Doc, action);
   return rc;
 }
 
@@ -388,9 +390,8 @@ static int UdmDocUpdate(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
   int    rc=UDM_OK;
   UDM_RESULT  *I = &Indexer->Indexed;
 
-  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+  UDM_LOCK_CHECK_OWNER(Indexer, UDM_LOCK_CONF);
   maxsize = UdmVarListFindInt(&Indexer->Conf->Vars,"DocMemCacheSize",0) * 1024 * 1024;
-  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
 
   if (maxsize > 0 && I->memused > 0) UdmLog(Indexer, UDM_LOG_EXTRA, "DocCacheSize: %d/%d", I->memused, maxsize);
   if (Doc)
@@ -449,21 +450,22 @@ static int UdmDocUpdate(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
   return rc;
 }
 
-__C_LINK int __UDMCALL UdmURLAction(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd) {
+int UdmURLActionNoLock(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd)
+{
   int res=UDM_ERROR, execflag = 0;
   size_t i, dbfrom = 0, dbto;
   UDM_DB  *db;
   int dbnum = (cmd == UDM_URL_ACTION_GET_CACHED_COPY ? UdmVarListFindInt(&D->Sections, "dbnum", 0) : -1);
 
+  UDM_LOCK_CHECK_OWNER(A, UDM_LOCK_CONF);
+
   if(cmd == UDM_URL_ACTION_FLUSH)
     return UdmDocUpdate(A, D);
 
-
 #ifdef USE_TRACE
   fprintf(A->TR, "[%d] URLAction: %d\n", A->handle, cmd);
 #endif
   
-  UDM_GETLOCK(A, UDM_LOCK_CONF);
   dbto =  A->Conf->dbl.nitems;
   if (dbnum < 0 && D != NULL)
   {
@@ -471,7 +473,6 @@ __C_LINK int __UDMCALL UdmURLAction(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd) {
     dbfrom = dbto = ((url_id) ? url_id : UdmStrHash32(UdmVarListFindStr(&D->Sections, "URL", ""))) % A->Conf->dbl.nitems;
     dbto++;
   }
-  UDM_RELEASELOCK(A, UDM_LOCK_CONF);
 
   for (i = dbfrom; i < dbto; i++)
   {
@@ -515,16 +516,29 @@ __C_LINK int __UDMCALL UdmURLAction(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd) {
 }
 
 
+/*
+  URL action with automatic UDM_LOCK_CONF locking
+*/
+__C_LINK int __UDMCALL UdmURLAction(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd)
+{
+  int rc;
+  UDM_GETLOCK(A, UDM_LOCK_CONF);
+  rc= UdmURLActionNoLock(A, D, cmd);
+  UDM_RELEASELOCK(A, UDM_LOCK_CONF);
+  return rc;
+}
+
+
+
 __C_LINK int __UDMCALL UdmTargets(UDM_AGENT *A)
 {
   int  res=UDM_ERROR;
   UDM_DB  *db;
   size_t i, dbfrom = 0, dbto;
 
-  UDM_GETLOCK(A, UDM_LOCK_CONF);
+  UDM_LOCK_CHECK_OWNER(A, UDM_LOCK_CONF);
   dbto =  A->Conf->dbl.nitems;
   UdmResultFree(&A->Conf->Targets);
-  UDM_RELEASELOCK(A, UDM_LOCK_CONF);
 
   for (i = dbfrom; i < dbto; i++)
   {
@@ -611,7 +625,6 @@ __C_LINK int __UDMCALL UdmSrvAction(UDM_AGENT *A, UDM_SERVERLIST *S, int cmd)
   
   UDM_GETLOCK(A, UDM_LOCK_CONF);
   dbto =  A->Conf->dbl.nitems;
-  UDM_RELEASELOCK(A, UDM_LOCK_CONF);
 
   strcpy(A->Conf->errstr, "No appropriate storage support compiled");
   for (i = dbfrom; i < dbto; i++)
@@ -628,6 +641,7 @@ __C_LINK int __UDMCALL UdmSrvAction(UDM_AGENT *A, UDM_SERVERLIST *S, int cmd)
     UDM_RELEASELOCK(A, UDM_LOCK_DB);
     if (res != UDM_OK) break;
   }
+  UDM_RELEASELOCK(A, UDM_LOCK_CONF);
   return res;
 }
 
@@ -648,9 +662,14 @@ int UdmFindWordsDB(UDM_AGENT *A, UDM_RESULT *CurRes, UDM_DB *db)
       break;
 #ifdef HAVE_SQL
     default:
-      rc= UdmFindWordsSQL(A, CurRes, db);
-      if (rc == UDM_OK && CurRes->CoordList.ncoords)
-        rc= UdmUserCacheStoreSQL(A, CurRes, db);
+      if (UDM_OK != (rc= UdmQueryCacheGetSQL(A, CurRes, db)))
+        return rc;
+      if (!CurRes->CoordList.ncoords)
+      {
+        rc= UdmFindWordsSQL(A, CurRes, db);
+        if (rc == UDM_OK && CurRes->CoordList.ncoords)
+          rc= UdmQueryCachePutSQL(A, CurRes, db);
+      }
       break;
 #endif
   }
@@ -863,21 +882,19 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   
   Res=UdmResultInit(NULL);
   UdmPrepare(A, Res);
-  if (!Res->WWList.nwords)
-    goto ret;
-  
   UdmVarListAddStr(&A->Conf->Vars, "orig_m", UdmVarListFindStr(&A->Conf->Vars, "m", "all"));
 
-  if (UDM_OK != (res= UdmFindWords(A, Res)))
-      goto ret;
-
+  if (!Res->WWList.nwords ||
+      UDM_OK != (res= UdmFindWords(A, Res)))
+    goto conv;
+  
   if (!Res->total_found)
   {
     int suggest= UdmVarListFindBool(&A->Conf->Vars, "Suggest", 0);
     if (suggest)
     {
       if(UDM_OK != (res= UdmResAction(A, Res, UDM_RES_ACTION_SUGGEST)))
-        goto ret;
+        goto conv;
     }
   }
   
@@ -989,7 +1006,6 @@ conv:
   ticks_ = UdmStartTimer() - ticks_;
   UdmLog(A, UDM_LOG_DEBUG, "Stop  Excerpts:\t\t%.2f", (float)ticks_/1000);
 
-ret:  
   UdmResWordInfo(A->Conf, Res);
   Res->work_time=ticks=UdmStartTimer()-ticks;
   UdmLog(A,UDM_LOG_DEBUG,"Done  UdmFind %.2f",(float)ticks/1000);
@@ -1023,6 +1039,7 @@ __C_LINK const char* __UDMCALL UdmDBTypeToStr(int dbtype)
     case UDM_DB_MSSQL:   return "mssql";
     case UDM_DB_ORACLE8: return "oracle";
     case UDM_DB_SQLITE:  return "sqlite";
+    case UDM_DB_SQLITE3: return "sqlite";
     case UDM_DB_MIMER:   return "mimer";
     case UDM_DB_VIRT:    return "virtuoso";
     case UDM_DB_ACCESS:  return "access";
@@ -1144,6 +1161,14 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     &udm_sqldb_sqlite_handler,
   },
 #endif
+#if (HAVE_SQLITE3)
+  {
+    "sqlite3",
+    UDM_DB_SQLITE3, UDM_DB_SQLITE3, 1,
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY,
+    &udm_sqldb_sqlite3_handler,
+  },
+#endif
 #if (HAVE_ODBC)
   {
     "odbc-solid", UDM_DB_SOLID, UDM_DB_ODBC, 1,
@@ -1367,7 +1392,9 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
       return UDM_ERROR;
   }
 
-  if(db->DBDriver==UDM_DB_IBASE || db->DBDriver==UDM_DB_SQLITE)
+  if(db->DBDriver == UDM_DB_IBASE ||
+     db->DBDriver == UDM_DB_SQLITE ||
+     db->DBDriver == UDM_DB_SQLITE3)
   {
     /*
       Ibase is a special case:
@@ -1400,8 +1427,10 @@ __C_LINK int __UDMCALL UdmStatAction(UDM_AGENT *A, UDM_STATLIST *S)
 {
   UDM_DB  *db;
   int  res=UDM_ERROR;
-  size_t i, dbfrom = 0, dbto =  A->Conf->dbl.nitems;
-  
+  size_t i, dbfrom = 0, dbto;
+
+  UDM_GETLOCK(A, UDM_LOCK_CONF);
+  dbto=  A->Conf->dbl.nitems;
   S->nstats = 0;
   S->Stat = NULL;
 
@@ -1419,6 +1448,7 @@ __C_LINK int __UDMCALL UdmStatAction(UDM_AGENT *A, UDM_STATLIST *S)
   {
     strcpy(A->Conf->errstr,db->errstr);
   }
+  UDM_RELEASELOCK(A, UDM_LOCK_CONF);
   return res;
 }
 
@@ -1499,7 +1529,6 @@ int UdmCheckUrlid(UDM_AGENT *Agent, urlid_t id)
 
   UDM_GETLOCK(Agent, UDM_LOCK_CONF);
   dbto =  Agent->Conf->dbl.nitems;
-  UDM_RELEASELOCK(Agent, UDM_LOCK_CONF);
 
   for (i = dbfrom; i < dbto; i++)
   {
@@ -1516,6 +1545,7 @@ int UdmCheckUrlid(UDM_AGENT *Agent, urlid_t id)
     UDM_RELEASELOCK(Agent, UDM_LOCK_DB);
     if (rc != 0) break;
   }
+  UDM_RELEASELOCK(Agent, UDM_LOCK_CONF);
   return rc;
 }
 
@@ -1623,6 +1653,7 @@ int UdmWordCacheFlush (UDM_AGENT *Indexer)
 {
 #ifdef HAVE_SQL
   size_t i;
+  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
   for (i= 0; i < Indexer->Conf->dbl.nitems; i++)
   {
     int rc= UDM_OK;
@@ -1640,6 +1671,7 @@ int UdmWordCacheFlush (UDM_AGENT *Indexer)
     }
     UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
   }
+  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
 #endif
   return UDM_OK;
 }
@@ -1764,6 +1796,15 @@ int UdmExport (UDM_AGENT *Indexer)
   return(0);
 }
 
+
+static int mccmpword(uint4 *a, uint4 *b)
+{
+  if (*a > *b) return 1;
+  if (*a < *b) return -1;
+  return 0;
+}
+
+
 static size_t UdmMultiCacheAddWord (UDM_MULTI_CACHE_WORD *cache, uint4 coord)
 {
   uint4 *tmp;
@@ -1774,6 +1815,7 @@ static size_t UdmMultiCacheAddWord (UDM_MULTI_CACHE_WORD *cache, uint4 coord)
   cache->intags = tmp;
   cache->intags[cache->nintags] = coord;
   cache->nintags++;
+  UdmSort(cache->intags, cache->nintags, sizeof(uint4), (udm_qsort_cmp)mccmpword);
   return(1);
 }
 
