@@ -115,6 +115,8 @@ static int UdmCheckDocPerSite(UDM_AGENT *Indexer, UDM_HREF *Href, size_t *res,
                               const char *hostinfo, size_t hostinfo_length)
 {
   int rc= UDM_OK;
+
+  UDM_LOCK_CHECK_OWNER(Indexer, UDM_LOCK_CONF);
   *res= UdmDocPerSiteCached(Indexer, hostinfo, hostinfo_length);
 
   if (*res < Href->max_doc_per_site)
@@ -123,7 +125,7 @@ static int UdmCheckDocPerSite(UDM_AGENT *Indexer, UDM_HREF *Href, size_t *res,
     UDM_DOCUMENT Doc;
     UdmDocInit(&Doc);
     UdmVarListAddStr(&Doc.Sections, "Hostinfo", hostinfo);
-    rc= UdmURLAction(Indexer, &Doc, UDM_URL_ACTION_DOCPERSITE);
+    rc= UdmURLActionNoLock(Indexer, &Doc, UDM_URL_ACTION_DOCPERSITE);
     doc_per_site_db= UdmVarListFindInt(&Doc.Sections, "DocPerSite", 0);
     UdmDocFree(&Doc);
     *res+= doc_per_site_db;
@@ -197,7 +199,7 @@ __C_LINK int __UDMCALL UdmStoreHrefs(UDM_AGENT * Indexer)
       UdmVarListReplaceInt(&Doc.Sections,"Site_id", H->site_id);
       UdmVarListReplaceInt(&Doc.Sections,"Server_id", H->server_id);
       UdmVarListReplaceInt(&Doc.Sections, "HTDB_URL_ID", H->rec_id);
-      if(UDM_OK != (rc= UdmURLAction(Indexer, &Doc, action)))
+      if(UDM_OK != (rc= UdmURLActionNoLock(Indexer, &Doc, action)))
         goto ret;
       UdmVarListFree(&Doc.Sections);
       H->stored= 1;
@@ -412,6 +414,8 @@ int UdmConvertHref(UDM_AGENT *Indexer, UDM_URL *CurURL,
   UDM_SERVER  *Srv;
   char    reason[1024]="";
   int rc= UDM_OK;
+
+  UDM_LOCK_CHECK_OWNER(Indexer, UDM_LOCK_CONF);
   
   UdmURLInit(&newURL);
 
@@ -451,10 +455,9 @@ int UdmConvertHref(UDM_AGENT *Indexer, UDM_URL *CurURL,
     
   UdmURLParse(&newURL, newhref);
   
+  Href->method= UDM_METHOD_GET;
   Href->site_id = 0;
-  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
   if(!(Srv = UdmServerFind(Indexer->Conf, &Indexer->Conf->Servers, newhref, NULL))) {
-    UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
     UdmLog(Indexer,UDM_LOG_DEBUG,"no Server, skip it",newhref);
     Href->method=UDM_METHOD_DISALLOW;
     goto ret;
@@ -470,7 +473,6 @@ int UdmConvertHref(UDM_AGENT *Indexer, UDM_URL *CurURL,
       Href->method= UDM_METHOD_DISALLOW;
     }
   }
-  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
 
   if (Href->method == UDM_METHOD_DISALLOW)
     goto ret;
@@ -527,6 +529,8 @@ static int UdmDocConvertHrefs(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
   urlid_t url_id = (urlid_t)UdmVarListFindInt(&Doc->Sections, "ID", 0);
   uint4   maxhops = UdmVarListFindUnsigned(&Doc->Sections, "MaxHops", 255);
 
+  UDM_LOCK_CHECK_OWNER(Indexer, UDM_LOCK_CONF);
+  
   for(i=0;i<Doc->Hrefs.nhrefs;i++){
     UDM_HREF  *Href=&Doc->Hrefs.Href[i];
     Href->hops=hops+1;
@@ -551,9 +555,10 @@ int UdmDocStoreHrefs(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
     return UDM_OK;
 
   UdmDocBaseHref(Indexer,Doc);
+  
+  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);  
   UdmDocConvertHrefs(Indexer,Doc);
 
-  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);  
   for(i=0;i<Doc->Hrefs.nhrefs;i++)
   {
     UDM_HREF  *Href=&Doc->Hrefs.Href[i];
@@ -567,11 +572,13 @@ int UdmDocStoreHrefs(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
 
 /*********************** 'UrlFile' stuff (for -f option) *******************/
 
-__C_LINK int __UDMCALL UdmURLFile(UDM_AGENT *Indexer, const char *fname,int action){
+__C_LINK int __UDMCALL UdmURLFile(UDM_AGENT *Indexer,
+                                  const char *fname, int action)
+{
   FILE *url_file;
   char str[1024]="";
   char str1[1024]="";
-  int result,res;
+  int result= UDM_OK, res;
   UDM_URL myurl;
   UDM_HREF Href;
 
@@ -580,6 +587,8 @@ __C_LINK int __UDMCALL UdmURLFile(UDM_AGENT *Indexer, const char *fname,int acti
 #endif
   UdmURLInit(&myurl);
   
+  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+  
   /* Read lines and clear/insert/check URLs                     */
   /* We've already tested in main.c to make sure it can be read */
   /* FIXME !!! Checking should be done here surely              */
@@ -609,18 +618,16 @@ __C_LINK int __UDMCALL UdmURLFile(UDM_AGENT *Indexer, const char *fname,int acti
     switch(action){
     case UDM_URL_FILE_REINDEX:
       UdmVarListReplaceStr(&Indexer->Conf->Vars, "ul", str);
-      result = UdmURLAction(Indexer, NULL, UDM_URL_ACTION_EXPIRE);
-      if(result!=UDM_OK) { UdmURLFree(&myurl); 
-      UdmViolationExit(paran);
-      return(result); }
+      result = UdmURLActionNoLock(Indexer, NULL, UDM_URL_ACTION_EXPIRE);
+      if(result!=UDM_OK)
+        goto ex;
       UdmVarListDel(&Indexer->Conf->Vars, "ul");
       break;
     case UDM_URL_FILE_CLEAR:
       UdmVarListReplaceStr(&Indexer->Conf->Vars, "ul", str);
       result=UdmClearDatabase(Indexer);
-      if(result!=UDM_OK) { UdmURLFree(&myurl); 
-      UdmViolationExit(paran);
-      return(UDM_ERROR); }
+      if(result!=UDM_OK)
+        goto ex;
       UdmVarListDel(&Indexer->Conf->Vars, "ul");
       break;
     case UDM_URL_FILE_INSERT:
@@ -642,9 +649,8 @@ __C_LINK int __UDMCALL UdmURLFile(UDM_AGENT *Indexer, const char *fname,int acti
         default:
           UdmLog(Indexer,UDM_LOG_ERROR,"Error in URL: '%s'",str);
         }
-        UdmURLFree(&myurl);
-        UdmViolationExit(paran);
-        return(UDM_ERROR);
+        result= UDM_ERROR;
+        goto ex;
       }
       break;
     }
@@ -652,9 +658,11 @@ __C_LINK int __UDMCALL UdmURLFile(UDM_AGENT *Indexer, const char *fname,int acti
   }
   if(url_file!=stdin)
     fclose(url_file);
+ex:
+  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
   UdmURLFree(&myurl);
   UdmViolationExit(paran);
-  return(UDM_OK);
+  return result;
 }
 
 
@@ -787,33 +795,36 @@ static void UdmAppendTarget(UDM_AGENT *Indexer, const char *url, const char *lan
 
   UDM_GETLOCK(Indexer, UDM_LOCK_THREAD);
   UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
-  if (Indexer->Conf->Targets.num_rows > 0) {
-    for (i = Indexer->Conf->Targets.num_rows - 1; i > 0; i--) {
-      Doc = &Indexer->Conf->Targets.Doc[i];
-      if (strcasecmp(UdmVarListFindStr(&Doc->Sections, "URL", ""), url) == 0 
-    && strcmp(UdmVarListFindStr(&Doc->RequestHeaders, "Accept-Language", ""), lang) == 0) {
-  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
-  UDM_RELEASELOCK(Indexer, UDM_LOCK_THREAD);
-  return;
-      }
+  
+  if (Indexer->Conf->Targets.num_rows > 0)
+  {
+    for (i= Indexer->Conf->Targets.num_rows - 1; i > 0; i--)
+    {
+      Doc= &Indexer->Conf->Targets.Doc[i];
+      if (strcasecmp(UdmVarListFindStr(&Doc->Sections, "URL", ""), url) == 0 &&
+          strcmp(UdmVarListFindStr(&Doc->RequestHeaders, "Accept-Language", ""), lang) == 0)
+        goto ret;
     }
   }
-  if ((Indexer->Conf->Targets.Doc = 
-       UdmRealloc(Save = Indexer->Conf->Targets.Doc, (Indexer->Conf->Targets.num_rows + 1) * sizeof(UDM_DOCUMENT))) == NULL) {
-    Indexer->Conf->Targets.Doc = Save;
-    UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
-    UDM_RELEASELOCK(Indexer, UDM_LOCK_THREAD);
-    return;
+  if ((Indexer->Conf->Targets.Doc= 
+       UdmRealloc(Save= Indexer->Conf->Targets.Doc,
+                  (Indexer->Conf->Targets.num_rows + 1) *
+                  sizeof(UDM_DOCUMENT))) == NULL)
+  {
+    Indexer->Conf->Targets.Doc= Save;
+    goto ret;
   }
-  Doc = &Indexer->Conf->Targets.Doc[Indexer->Conf->Targets.num_rows++];
-  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
+  Doc= &Indexer->Conf->Targets.Doc[Indexer->Conf->Targets.num_rows++];
   UdmDocInit(Doc);
   UdmVarListAddStr(&Doc->Sections, "URL", url);
   UdmVarListAddInt(&Doc->Sections, "Hops", hops);
   UdmVarListReplaceInt(&Doc->Sections, "URL_ID", UdmStrHash32(url));
   UdmVarListReplaceInt(&Doc->Sections, "Referrer-ID", parent);
-  UdmURLAction(Indexer, Doc, UDM_URL_ACTION_ADD);
+  UdmURLActionNoLock(Indexer, Doc, UDM_URL_ACTION_ADD);
   if (*lang != '\0') UdmVarListAddStr(&Doc->RequestHeaders, "Accept-Language", lang);
+  
+ret:
+  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
   UDM_RELEASELOCK(Indexer, UDM_LOCK_THREAD);
   return;
 }
@@ -1527,12 +1538,13 @@ static int
 UdmDocImportSections(UDM_AGENT *A, UDM_DOCUMENT *D, UDM_CHARSET *doccs)
 {
   int rc= UDM_OK;
+  UDM_LOCK_CHECK_OWNER(A, UDM_LOCK_CONF);
   if (doccs && doccs != D->lcs)
   {
     UDM_DOCUMENT IDoc;
     UdmDocInit(&IDoc);
     UdmVarListReplaceLst(&IDoc.Sections, &D->Sections, NULL, "*");
-    if (UDM_OK != (rc= UdmURLAction(A, &IDoc, UDM_URL_ACTION_SQLIMPORTSEC)))
+    if (UDM_OK != (rc= UdmURLActionNoLock(A, &IDoc, UDM_URL_ACTION_SQLIMPORTSEC)))
       goto ex;
     if (IDoc.TextList.nitems)
     {
@@ -1560,7 +1572,7 @@ UdmDocImportSections(UDM_AGENT *A, UDM_DOCUMENT *D, UDM_CHARSET *doccs)
   }
   else
   {
-    if (UDM_OK != (rc= UdmURLAction(A, D, UDM_URL_ACTION_SQLIMPORTSEC)))
+    if (UDM_OK != (rc= UdmURLActionNoLock(A, D, UDM_URL_ACTION_SQLIMPORTSEC)))
       goto ex;
   }
 ex:
@@ -1912,13 +1924,13 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
 
         /* Check whether URL is disallowed by robots.txt */
         UDM_GETLOCK(Indexer,UDM_LOCK_CONF);
-        rule = UdmRobotRuleFind(&Indexer->Conf->Robots, &Doc.CurURL);
-        UDM_RELEASELOCK(Indexer,UDM_LOCK_CONF);
-        if(rule) {
+        if((rule= UdmRobotRuleFind(&Indexer->Conf->Robots, &Doc.CurURL)))
+        {
           UdmLog(Indexer,UDM_LOG_WARN,"robots.txt: '%s %s'",(rule->cmd==UDM_METHOD_DISALLOW)?"Disallow":"Allow",rule->path);
           if(rule->cmd==UDM_METHOD_DISALLOW)
             Doc.method=rule->cmd;
         }
+        UDM_RELEASELOCK(Indexer,UDM_LOCK_CONF);
       }
     }
     if(origurl != NULL){
@@ -2043,7 +2055,10 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
         UdmParseHeaders(Indexer,&Doc);
       }
       
-      if (UDM_OK != (result= UdmDocImportSections(Indexer, &Doc, doccs)))
+      UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+      result= UdmDocImportSections(Indexer, &Doc, doccs);
+      UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
+      if (UDM_OK != result)
         goto ex;
 
       /* Create "HTTP.LocalCharsetContent" variable if requested */
@@ -2124,7 +2139,11 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
         }  
       }
       UDM_RELEASELOCK(Indexer,UDM_LOCK_CONF);
-      if(UDM_OK != (result = UdmURLAction(Indexer, &Doc, UDM_URL_ACTION_LINKS_DELETE)))
+      
+      UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+      result = UdmURLActionNoLock(Indexer, &Doc, UDM_URL_ACTION_LINKS_DELETE);
+      UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
+      if(UDM_OK != result)
       {
         UdmViolationExit(paran);
         return result;
@@ -2177,7 +2196,9 @@ ex:
   UDM_FREE(Doc.Buf.buf);
   Doc.Buf.maxsize=0;
   
-  result = UdmURLAction(Indexer, &Doc, UDM_URL_ACTION_FLUSH);
+  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+  result= UdmURLActionNoLock(Indexer, &Doc, UDM_URL_ACTION_FLUSH);
+  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
   
   UDM_FREE(origurl); UDM_FREE(aliasurl);
   UDM_GETLOCK(Indexer, UDM_LOCK_THREAD);
