@@ -501,6 +501,8 @@ static UDM_WIDEWORDLIST *UdmAllForms1(UDM_AGENT *Indexer,
   {
     size_t nbytes;
     w.order= uword->order;
+    w.phrpos= uword->phrpos;
+    w.phrlen= uword->phrlen;
     w.count= 0;
     w.origin= UDM_WORD_ORIGIN_SPELL;
     w.word= *R;
@@ -644,6 +646,8 @@ static int UdmAllFormsTranslit(UDM_AGENT *A, UDM_WIDEWORDLIST *result,
     UdmConvInit(&uni_lcs, &udm_charset_sys_int, A->Conf->lcs, UDM_RECODE_HTML);
     UdmConv(&uni_lcs, lcsword, sizeof(lcsword), (const char *) tword, nbytes);
     w.order= uword->order;
+    w.phrpos= uword->phrpos;
+    w.phrlen= uword->phrlen;
     w.count= 0;
     w.origin= UDM_WORD_ORIGIN_SYNONYM;
     w.word= lcsword;
@@ -681,6 +685,8 @@ UdmAllForms2(UDM_AGENT *Indexer,
       nrows= UdmSQLNumRows(&SQLRes);
       bzero((void*) &form, sizeof(UDM_WIDEWORD));
       form.order= uw->order;
+      form.phrpos= uw->phrpos;
+      form.phrlen= uw->phrlen;
       form.count= 0;
       form.uword= uword;
       form.origin= UDM_WORD_ORIGIN_SYNONYM;
@@ -764,25 +770,247 @@ static UDM_WIDEWORDLIST *UdmAllForms(UDM_AGENT *Indexer,
   return result;
 }
 
+
+static int
+UdmAutoPhraseChar(int ch)
+{
+  return ch == '_' || ch == '-' || ch == '.' || ch == '/';
+}
+
+
+static int
+UdmStackItemAdd(UDM_RESULT *Res, UDM_STACK_ITEM *item)
+{
+  if (Res->nitems >= Res->mitems)
+  {
+    Res->mitems+= UDM_MAXSTACK;
+    if (!(Res->items= (UDM_STACK_ITEM*)UdmRealloc(Res->items,
+                                                  Res->mitems *
+                                                  sizeof(UDM_STACK_ITEM))))
+      return UDM_ERROR;
+  }
+  Res->items[Res->nitems]= item[0];
+  Res->nitems++;
+  return UDM_OK;
+}
+
+
+typedef struct udm_stack_parser_state_st
+{
+  int secno;
+  int next_word_match_type;
+  int use_numeric_operators;
+  int word_match;
+  int nphrasecmd;
+  int auto_phrase;
+  int phrpos;
+  const char *lang;
+} UDM_STACK_PARSER_STATE;
+
+
+
+static int
+UdmStackItemAddCmd(UDM_RESULT *Res,
+                   UDM_STACK_PARSER_STATE *state,
+                   int *lex, size_t length)
+{
+  int rc= UDM_OK;
+  size_t i;
+  UDM_STACK_ITEM item;
+  
+  for (i = 0; i < length; i++)
+  {
+     switch(lex[i])
+     {
+       case '&':
+       case '+':
+         item.cmd= UDM_STACK_AND;
+         state->next_word_match_type= state->word_match;
+         break;
+       case '|':
+         item.cmd= UDM_STACK_OR;
+         state->next_word_match_type= state->word_match;
+         break;
+       case '~':
+         item.cmd= UDM_STACK_NOT;
+         state->next_word_match_type= state->word_match;
+         break;
+       case '(':
+         item.cmd= UDM_STACK_LEFT;
+         state->next_word_match_type= state->word_match;
+         break;
+       case ')':
+         item.cmd= UDM_STACK_RIGHT;
+         state->next_word_match_type= state->word_match;
+         break;
+       case '"':
+         item.cmd= UDM_STACK_PHRASE;
+         state->next_word_match_type= state->word_match;
+         state->nphrasecmd++;
+         break;
+       case '<':
+         state->next_word_match_type= state->use_numeric_operators ?
+                                      UDM_MATCH_NUMERIC_LT : state->word_match;
+         break;
+       case '>':
+         state->next_word_match_type= state->use_numeric_operators ? 
+                                      UDM_MATCH_NUMERIC_GT : state->word_match;
+         break;
+       default:
+         if (state->auto_phrase && !UdmAutoPhraseChar(lex[i]))
+         {
+           int quot= '"';
+           item.cmd= UDM_STACK_PHRASE;
+           item.arg= 0;
+           UdmStackItemAddCmd(Res, state, &quot, 1);
+           state->auto_phrase= 0;
+         }
+         continue;
+     }
+     /* Ignore all operators if we are in a phrase,  except phrase end. */
+    if (!(state->nphrasecmd % 2) || item.cmd == UDM_STACK_PHRASE)
+    {
+      item.arg= 0;
+      rc= UdmStackItemAdd(Res, &item);
+      Res->ncmds++;
+    }
+  }
+  return rc;
+}
+
+
+static int
+UdmStackItemAddWord(UDM_AGENT *query, UDM_RESULT *Res,
+                   UDM_STACK_PARSER_STATE *state,
+                   int *lex, size_t length, int *lt,
+                   int *uwrd, char *wrd, UDM_CONV *uni_lc)
+{
+  UDM_WIDEWORD OWord;
+  UDM_WIDEWORDLIST Forms;
+  UDM_STACK_ITEM item;
+  int origin, rc, lt_auto_phrase= UdmAutoPhraseChar(lt[0]);
+  size_t phrlen= 0;
+
+  if(Res->WWList.nuniq >= UDM_MAXWORDPERQUERY-1)
+    return UDM_OK;
+
+  if (state->nphrasecmd % 2) /* in phrase */
+  {
+    if (((state->auto_phrase && !lt_auto_phrase) || lt[0] == '"'))
+    {
+      /* End of auto- or non-auto-phrase*/
+      phrlen= state->phrpos + 1;
+    }
+  }
+  else /* not in phrase */
+  {
+    if (lt_auto_phrase)
+    {
+      /* Start of auto-phrase */
+      int quot= '"';
+      state->auto_phrase= 1;
+      item.cmd= UDM_STACK_PHRASE;
+      item.arg= 0;
+      UdmStackItemAddCmd(Res, state, &quot, 1);
+      phrlen= 0;
+    }
+    else
+      phrlen= 1; /* Single word */
+  }
+  
+  item.cmd= UDM_STACK_WORD;
+  item.arg= Res->WWList.nuniq;
+  rc= UdmStackItemAdd(Res, &item);
+
+
+  /*
+    Check stopword only when full word.
+    Substring searches should not exclude them.
+  */
+  if(state->word_match == UDM_MATCH_FULL &&
+     (UdmStopListFind(&query->Conf->StopWords, wrd, state->lang) ||
+      query->Conf->WordParam.min_word_len > length ||
+      query->Conf->WordParam.max_word_len < length))
+  {
+    origin= UDM_WORD_ORIGIN_STOP;
+    Res->items[Res->nitems - 1].cmd= UDM_STACK_STOP;
+  }
+  else
+  {
+    origin= UDM_WORD_ORIGIN_QUERY;
+  }
+
+  OWord.len= strlen(wrd);
+  OWord.order= Res->WWList.nuniq;
+  OWord.phrpos= state->phrpos;
+  OWord.phrlen= phrlen;
+  OWord.count= 0;
+  OWord.word= wrd;
+  OWord.uword= uwrd;
+  OWord.origin = origin;
+  OWord.match= state->next_word_match_type;
+  OWord.secno= state->secno;
+  UdmWideWordListAdd(&Res->WWList, &OWord);
+  if (state->nphrasecmd % 2)
+    state->phrpos++;
+      
+  if (origin == UDM_WORD_ORIGIN_STOP)
+    return UDM_OK;
+      
+  UdmWideWordListInit(&Forms);
+  if(UdmAllForms(query,&Forms,&OWord))
+  {
+    UDM_WIDEWORD FWord;
+    size_t frm;
+    for (frm= 0; frm < Forms.nwords ; frm++)
+    {
+      UdmConv(uni_lc,wrd,12*query->Conf->WordParam.max_word_len,
+             (char*)(Forms.Word[frm].uword),
+              sizeof(Forms.Word[frm].uword[0])*(UdmUniLen(Forms.Word[frm].uword)+1));
+      FWord.len= strlen(wrd);
+      FWord.order= Res->WWList.nuniq;
+      FWord.phrpos= state->phrpos;
+      FWord.phrlen= phrlen;
+      FWord.count= 0;
+      FWord.word= wrd;
+      FWord.uword= Forms.Word[frm].uword;
+      FWord.origin = Forms.Word[frm].origin;
+      FWord.match= state->next_word_match_type;
+      FWord.secno= state->secno;
+      UdmWideWordListAdd(&Res->WWList,&FWord);
+/*    UdmLog(query, UDM_LOG_DEBUG, "Word form: [%d] %s", FWord.origin, wrd);*/
+    }
+  }
+  UdmWideWordListFree(&Forms);
+  Res->WWList.nuniq++;
+  return rc;
+}
+
+
 int UdmPrepare(UDM_AGENT * query,UDM_RESULT *Res)
 {
   UDM_CHARSET * browser_cs, * local_cs, *sys_int;
   int  ctype;
   int * ustr, * lt, * lex;
   size_t ulen;
-  int word_match   = UdmMatchMode(UdmVarListFindStr(&query->Conf->Vars, "wm", "wrd"));
-/*int search_mode = UdmSearchMode(UdmVarListFindStr(&query->Conf->Vars, "m", "all")); */
-  int use_numeric_operators= UdmVarListFindBool(&query->Conf->Vars, "UseNumericOperators", 0);
   const char * txt = UdmVarListFindStr(&query->Conf->Vars,"q","");
   const char * qprev = UdmVarListFindStr(&query->Conf->Vars,"qprev","");
-  const char * qlang = UdmVarListFindStr(&query->Conf->Vars, "g", NULL);
   char *ltxt;
-  size_t i, wlen, llen, nphrasecmd = 0;
+  size_t wlen, llen;
   char *wrd, *clex;
   int *uwrd;
   UDM_CONV uni_lc, bc_uni, bc_lc;
   const char *lang;
-  int next_word_match_type= word_match;
+  UDM_STACK_PARSER_STATE state;
+  
+  state.secno= 0;
+  state.use_numeric_operators= UdmVarListFindBool(&query->Conf->Vars, "UseNumericOperators", 0);
+  state.nphrasecmd= 0;
+  state.word_match= UdmMatchMode(UdmVarListFindStr(&query->Conf->Vars, "wm", "wrd"));
+  state.next_word_match_type= state.word_match;
+  state.lang= UdmVarListFindStr(&query->Conf->Vars, "g", NULL);
+  state.auto_phrase= 0;
+  state.phrpos= 0;
   
   if ((wrd = (char*)UdmMalloc(query->Conf->WordParam.max_word_len * 12 + 1)) == NULL) return 0;
   if ((uwrd = (int*)UdmMalloc(sizeof(int) * (query->Conf->WordParam.max_word_len + 1))) == NULL) { UDM_FREE(wrd); return 0; }
@@ -839,172 +1067,71 @@ int UdmPrepare(UDM_AGENT * query,UDM_RESULT *Res)
     uwrd[udm_min(wlen, query->Conf->WordParam.max_word_len)] = 0;
     UdmConv(&uni_lc, wrd, query->Conf->WordParam.max_word_len * 12,(char*)uwrd, sizeof(uwrd[0])*(wlen+1));
     clex = UdmTrim(wrd, " \t\r\n");
-      
-    if ((ctype != UDM_UNI_LETTER) && (ctype != UDM_UNI_CJK))
+    
+    if (ctype == UDM_UNI_SEPAR)
     {
-      for (i = 0; i < wlen; i++)
-      {
-         switch(lex[i])
-         {
-           case '&':
-           case '+':
-             Res->items[Res->nitems].cmd = UDM_STACK_AND;
-             next_word_match_type= word_match;
-             break;
-           case '|':
-             Res->items[Res->nitems].cmd = UDM_STACK_OR;
-             next_word_match_type= word_match;
-             break;
-           case '~':
-             Res->items[Res->nitems].cmd = UDM_STACK_NOT;
-             next_word_match_type= word_match;
-             break;
-           case '(':
-             Res->items[Res->nitems].cmd = UDM_STACK_LEFT;
-             next_word_match_type= word_match;
-             break;
-           case ')':
-             Res->items[Res->nitems].cmd = UDM_STACK_RIGHT;
-             next_word_match_type= word_match;
-             break;
-           case '"':
-             Res->items[Res->nitems].cmd = UDM_STACK_PHRASE;
-             next_word_match_type= word_match;
-             nphrasecmd++;
-             break;
-           case '<':
-             next_word_match_type= use_numeric_operators ?
-                                   UDM_MATCH_NUMERIC_LT : word_match;
-             break;
-           case '>':
-             next_word_match_type= use_numeric_operators ? 
-                                   UDM_MATCH_NUMERIC_GT : word_match;
-             break;
-           default: continue;
-         }
-	 if (! (nphrasecmd % 2) || Res->items[Res->nitems].cmd == UDM_STACK_PHRASE)
-	 {
-           Res->items[Res->nitems].arg=0;
-           Res->nitems++;
-           Res->ncmds++;
-           if (Res->nitems >= Res->mitems)
-           {
-             Res->mitems += UDM_MAXSTACK;
-             Res->items = (UDM_STACK_ITEM*)UdmRealloc(Res->items, Res->mitems * sizeof(UDM_STACK_ITEM));
-           }
-	 }
-      }
+      UdmStackItemAddCmd(Res, &state, lex, wlen);
     } 
     else
     {
-      UDM_WIDEWORD OWord;
-      UDM_WIDEWORDLIST Forms;
-      int origin;
+      UDM_VAR *Section;
 
-      if(Res->WWList.nuniq >= UDM_MAXWORDPERQUERY-1)
-        continue;
-
-      Res->items[Res->nitems].cmd=UDM_STACK_WORD;
-      Res->items[Res->nitems].arg = Res->WWList.nuniq;  /* 1L << (Res->WWList.nuniq); */
-      Res->nitems++;
-      if (Res->nitems >= Res->mitems)
+      if (lt[0] == ':')
       {
-        Res->mitems += UDM_MAXSTACK;
-        Res->items = (UDM_STACK_ITEM*)UdmRealloc(Res->items, Res->mitems * sizeof(UDM_STACK_ITEM));
-      }
-
-      {
-        size_t nw;
-        for (nw = 0; nw < Res->WWList.nwords; nw++)
+        if ((Section= UdmVarListFind(&query->Conf->Sections, wrd)))
         {
-          if (UdmUniStrCmp(Res->WWList.Word[nw].uword, uwrd)) continue;
-            continue;
+          state.secno= Section->section;
+          continue;
+        }
+        if (wlen > 5 && !strncmp(wrd, "secno", 5))
+        {
+          state.secno= atoi(wrd + 5);
+          continue;
         }
       }
-
-      /*
-        Check stopword only when full word.
-        Substring searches should not exclude them.
-      */
-      if(word_match == UDM_MATCH_FULL &&
-         (UdmStopListFind(&query->Conf->StopWords, wrd, qlang) ||
-          query->Conf->WordParam.min_word_len > wlen ||
-          query->Conf->WordParam.max_word_len < wlen))
-      {
-        origin= UDM_WORD_ORIGIN_STOP;
-        Res->items[Res->nitems - 1].cmd= UDM_STACK_STOP;
-      }
-      else
-      {
-        origin= UDM_WORD_ORIGIN_QUERY;
-      }
-
-      OWord.len= strlen(wrd);
-      OWord.order= Res->WWList.nuniq;
-      OWord.count= 0;
-      OWord.word= wrd;
-      OWord.uword= uwrd;
-      OWord.origin = origin;
-      OWord.match= next_word_match_type;
-      UdmWideWordListAdd(&Res->WWList, &OWord);
       
-      if (origin == UDM_WORD_ORIGIN_STOP)
-        continue;
-      
-      UdmWideWordListInit(&Forms);
-      if(UdmAllForms(query,&Forms,&OWord))
-      {
-        UDM_WIDEWORD FWord;
-        size_t frm;
-        for (frm= 0; frm < Forms.nwords ; frm++)
-        {
-          UdmConv(&uni_lc,wrd,12*query->Conf->WordParam.max_word_len,
-                  (char*)(Forms.Word[frm].uword),
-                   sizeof(Forms.Word[frm].uword[0])*(UdmUniLen(Forms.Word[frm].uword)+1));
-          FWord.len= strlen(wrd);
-          FWord.order= Res->WWList.nuniq;
-          FWord.count= 0;
-          FWord.word= wrd;
-          FWord.uword= Forms.Word[frm].uword;
-          FWord.origin = Forms.Word[frm].origin;
-          FWord.match= next_word_match_type;
-
-          UdmWideWordListAdd(&Res->WWList,&FWord);
-/*        UdmLog(query, UDM_LOG_DEBUG, "Word form: [%d] %s", FWord.origin, wrd);*/
-        }
-      }
-      UdmWideWordListFree(&Forms);
-      Res->WWList.nuniq++;
+      UdmStackItemAddWord(query, Res, &state, lex, wlen, lt, uwrd, wrd, &uni_lc);
     }
   }
   
-  if (nphrasecmd & 1)
+  if (state.nphrasecmd & 1)
   {
-    Res->items[Res->nitems].cmd = UDM_STACK_PHRASE;
-    Res->items[Res->nitems].arg=0;
-    Res->nitems++;
+    UDM_STACK_ITEM item;
+    item.cmd= UDM_STACK_PHRASE;
+    item.arg= 0;
+    UdmStackItemAdd(Res, &item);
     Res->ncmds++;
-    if (Res->nitems >= Res->mitems)
-    {
-      Res->mitems += UDM_MAXSTACK;
-      Res->items = (UDM_STACK_ITEM*)UdmRealloc(Res->items, Res->mitems * sizeof(UDM_STACK_ITEM));
-    }
   }
-  UDM_FREE(ustr); UDM_FREE(uwrd); UDM_FREE(wrd);
-  Res->WWList.wm = word_match;
-    
-  return(0);
+  UDM_FREE(ustr);
+  UDM_FREE(uwrd);
+  UDM_FREE(wrd);
+  Res->WWList.wm= state.word_match;
+  return UDM_OK;
 }
 
+
+typedef struct udm_score_param_st
+{
+  unsigned int *R;
+  unsigned int *D;
+  size_t ncosine;
+  size_t nsections;
+  float Rsum_factor;
+} UDM_SCORE_PARAM;
+
+
+
 /*
   R[i] and D[i] are in the range 0..64.
   ns is between 1..256
 */
 static
-inline uint4 UdmCalcCosineWeight(unsigned int *R, unsigned int *D, size_t num,
-                                 float Rsum_factor, float nwords_factor)
+inline uint4 UdmCalcCosineWeight(UDM_SCORE_PARAM *score_param,
+                                 float nwords_factor)
 {
-  size_t i;
+  size_t i, num= score_param->ncosine;
+  unsigned int *D= score_param->D;
+  unsigned int *R= score_param->R;
   float res;
 
   size_t Dsum=  D[0] * D[0];
@@ -1019,7 +1146,8 @@ inline uint4 UdmCalcCosineWeight(unsigned int *R, unsigned int *D, size_t num,
     }
   }
   
-  res= Rsum_factor * nwords_factor * (float) RDsum / sqrt(Dsum) + 0.5;
+  res= score_param->Rsum_factor * nwords_factor *
+       (float) RDsum / sqrt(Dsum) + 0.5;
 
 #if 0
   for (i=0; i < num; i++)
@@ -1041,46 +1169,46 @@ CalcAverageWordDistance(UDM_WIDEWORDLIST *WWL,
                         unsigned int dst_weight);
 
 static uint4
-UdmCalcCosineWeightQuick(unsigned int *R, unsigned int *D, size_t ncosine,
+UdmCalcCosineWeightQuick(UDM_SCORE_PARAM *score_param,
                          UDM_WIDEWORDLIST *WWL, 
                          UDM_URL_CRD *Crd, size_t ncoords,
-                         size_t nsections,
                          int *wf2, size_t nwf_num,
                          size_t dst_offs,
                          unsigned int dst_weight,
-                         float Rsum_factor, float nwords_factor)
+                         float nwords_factor)
 {
   float res;
   size_t Dsum= 0;
   size_t RDsum= 0;
   size_t add;
-  char *added= (char*) D;
+  char *added= (char*) score_param->D;
   
-  bzero(added, ncosine);
+  bzero(added, score_param->ncosine);
 
   add= CalcAverageWordDistance(WWL, Crd, ncoords, dst_weight);
   Dsum= add * add;
-  RDsum= R[dst_offs] * add;
+  RDsum= score_param->R[dst_offs] * add;
   
   for( ; ncoords; Crd++, ncoords--)
   {
     uint4 coord= Crd->coord;
     size_t wrdsec= UDM_WRDSEC(coord);
-    size_t wrdnum= UDM_WRDNUM(coord);
+    size_t wrdnum= WWL->Word[UDM_WRDNUM(coord)].order;
     UDM_WIDEWORD *W= &WWL->Word[wrdnum];
-    if (wrdsec < nsections)
+    if (wrdsec < score_param->nsections)
     {
-      size_t offs= nsections * wrdnum + wrdsec;
+      size_t offs= score_param->nsections * wrdnum + wrdsec;
       if (!added[offs])
       {
         add= wf2[wrdsec] + W->weight;
         Dsum+= add*add;
-        RDsum+= add * R[offs];
+        RDsum+= add * score_param->R[offs];
         added[offs]= 1;
       }
     }
   }
-  res= Rsum_factor * nwords_factor * (float) RDsum / sqrt(Dsum) + 0.5;
+  res= score_param->Rsum_factor * nwords_factor *
+        (float) RDsum / sqrt(Dsum) + 0.5;
 
   return (uint4) res;
 }
@@ -1448,6 +1576,7 @@ static void UdmGroupByURLNewRel(UDM_RESULT *Res)
 #define NUMWORD_FACTOR_SIZE 256
 #define NUMWORD_FACTOR(nwf, n) (nwf[n >= NUMWORD_FACTOR_SIZE ? NUMWORD_FACTOR_SIZE - 1 : n])
 #define MAXCOORD_FACTOR(factor, max_coord) ((1-((float)factor*(float)max_coord)))
+#define MINCOORD_FACTOR(factor, coord) ((float) 0x1000 / (float) (0x1000 + factor * (coord - 1)))
 
 static
 void UdmNumWordFactorInit(float k, float *numword_factor, size_t ncoords)
@@ -1497,7 +1626,7 @@ static void FillD(unsigned int *D,
   {
     uint4 coord= Crd->coord;
     size_t wrdsec= UDM_WRDSEC(coord);
-    size_t wrdnum= UDM_WRDNUM(coord);
+    size_t wrdnum= WWL->Word[UDM_WRDNUM(coord)].order;
     UDM_WIDEWORD *W= &WWL->Word[wrdnum];
     if (wrdsec < nsections)
     {
@@ -1516,8 +1645,8 @@ UdmGroupByURLInternal(UDM_AGENT *query,UDM_RESULT *Res, int search_mode)
   size_t  i, D_size, nitems = Res->nitems;
   size_t  *count, count_size = Res->WWList.nuniq * sizeof(size_t);
   size_t nsections = UdmVarListFindInt(&query->Conf->Vars, "NumSections", 256);
-  size_t dst_offs= Res->WWList.nwords * nsections;
-  size_t nwf_offs= Res->WWList.nwords * nsections + 1;
+  size_t dst_offs= Res->WWList.nuniq * nsections;
+  size_t nwf_offs= Res->WWList.nuniq * nsections + 1;
   size_t nwf_num= 0;  /* How many words in this section */
   size_t numcosine;
   int wf[256], wf2[256], nwf[256];
@@ -1528,17 +1657,21 @@ UdmGroupByURLInternal(UDM_AGENT *query,UDM_RESULT *Res, int search_mode)
   UDM_URL_CRD *CrdFrom, *CrdCurr;
   UDM_URL_CRD *CrdLast= Res->CoordList.Coords + Res->CoordList.ncoords;
   UDM_WIDEWORD *Res_WWList_Word= Res->WWList.Word;
-  float Rsum_factor, nword_factor;
+  UDM_SCORE_PARAM score_param;
+  float Rsum_factor;
   float numword_factor[NUMWORD_FACTOR_SIZE];
   float numwordfactor= ((float)UdmVarListFindDouble(&query->Conf->Vars,
                                                 "NumWordFactor", 25.5)) / 255;
   float max_coord_factor= ((float)UdmVarListFindInt(&query->Conf->Vars,
                                              "MaxCoordFactor", 255)) / 0xFFFFFF;
-  float min_coord_factor= ((float)UdmVarListFindInt(&query->Conf->Vars,
-                                             "MinCoordFactor", 0)) / 0xFFFFFF;
   unsigned int dst_weight= (unsigned int) UdmVarListFindInt(&query->Conf->Vars,
                                                  "WordDistanceWeight", 255);
 
+  unsigned int  MinCoordFactor= UdmVarListFindInt(&query->Conf->Vars,
+                                                  "MinCoordFactor", 0);
+  unsigned int numdistinctwordfactor= (unsigned int) UdmVarListFindInt(&query->Conf->Vars,
+                                                     "NumDistinctWordFactor", 0);
+  float numdistinctword_factor[NUMWORD_FACTOR_SIZE];
   if(!Res->CoordList.ncoords) return;
 
   if (0)
@@ -1547,6 +1680,13 @@ UdmGroupByURLInternal(UDM_AGENT *query,UDM_RESULT *Res, int search_mode)
     return;
   }
   
+  for (i= 0; i < Res->WWList.nuniq; i++)
+  {
+    unsigned int k= numdistinctwordfactor;
+    float x= (float) i / Res->WWList.nuniq;
+    numdistinctword_factor[i]= ((x * x * x * x * k) + (256-k)) / 256;
+  }
+  
   UdmWideWordListSetOriginWeight(&Res->WWList);
   UdmNumWordFactorInit(numwordfactor, numword_factor, Res->CoordList.ncoords);
   UdmWeightFactorsInit(UdmVarListFindStr(&query->Conf->Vars, "wf", ""), wf);
@@ -1558,7 +1698,7 @@ UdmGroupByURLInternal(UDM_AGENT *query,UDM_RESULT *Res, int search_mode)
     nwf_num= nsections;
   }  
 
-  numcosine= Res->WWList.nwords * nsections + 1 + nwf_num;
+  numcosine= Res->WWList.nuniq * nsections + 1 + nwf_num;
   
   D_size= numcosine * sizeof(unsigned int);
   
@@ -1576,22 +1716,16 @@ UdmGroupByURLInternal(UDM_AGENT *query,UDM_RESULT *Res, int search_mode)
   for(Rsum=0, i= 0; i < Res->WWList.nwords; i++)
   {
     size_t secno;
+    if (Res_WWList_Word[i].origin != UDM_WORD_ORIGIN_QUERY)
+      continue;
     for (secno= 0; secno < nsections; secno++)
     {
-      size_t offs= i * nsections + secno;
+      size_t offs= Res_WWList_Word[i].order * nsections + secno;
       R[offs] = wf2[secno] + Res_WWList_Word[i].weight;
       Rsum+= R[offs] * R[offs];
     }
   }
   
-#if 0
-  for (i= 0; i < nwf_num; i++)
-  {
-    size_t offs= nwf_offs + i;
-    R[offs]= nwf[i];
-    Rsum+= R[offs] * R[offs];
-  }
-#endif
 
   Rsum+= R[dst_offs] * R[dst_offs];
   Rsum_factor= 100000 / sqrt(Rsum);
@@ -1644,8 +1778,15 @@ UdmGroupByURLInternal(UDM_AGENT *query,UDM_RESULT *Res, int search_mode)
     nitems = j;
   }
 
+  score_param.R= R;
+  score_param.D= D;
+  score_param.ncosine= numcosine;
+  score_param.nsections= nsections;
+  score_param.Rsum_factor= Rsum_factor;
+
   for (CrdFrom= Crd ; CrdFrom < CrdLast ; CrdFrom= CrdCurr)
   {
+    float nword_factor;
     size_t z, phr_n;
     size_t max_coord= CrdFrom->coord;
     size_t min_coord= CrdFrom->coord;
@@ -1682,22 +1823,36 @@ UdmGroupByURLInternal(UDM_AGENT *query,UDM_RESULT *Res, int search_mode)
     }
     nword_factor= NUMWORD_FACTOR(numword_factor, phr_n) *
                   MAXCOORD_FACTOR(max_coord_factor, max_coord) *
-                  MAXCOORD_FACTOR(min_coord_factor, min_coord);
+                  MINCOORD_FACTOR(MinCoordFactor, min_coord);
+
+    if (search_mode != UDM_MODE_ALL)
+    {
+      size_t nuniq= 0;
+      for (z= 0; z < Res->WWList.nuniq; z++)
+      {
+        if (count[z])
+          nuniq++;
+      }
+      if (nuniq < Res->WWList.nuniq)
+        nword_factor= nword_factor * numdistinctword_factor[nuniq];
+    }
+
     if (nwf_num)
     {
       bzero((void*)D, D_size);
       FillD(D, &Res->WWList, CrdFrom, phr_n,
             nsections, wf2, nwf_num, D_nwf_offs, D_dst_offs, dst_weight);
       UdmNWFNormalize(D_nwf_offs, nwf, nwf_num);
-      CrdTo->coord= UdmCalcCosineWeight(R, D, numcosine, Rsum_factor, nword_factor);
+      CrdTo->coord= UdmCalcCosineWeight(&score_param, nword_factor);
     }
     else
     {
-      CrdTo->coord= UdmCalcCosineWeightQuick(R, D, numcosine,
-                                             &Res->WWList, CrdFrom, phr_n,
-                                             nsections, wf2, nwf_num,
+      CrdTo->coord= UdmCalcCosineWeightQuick(&score_param,
+                                             &Res->WWList,
+                                             CrdFrom, phr_n,
+                                             wf2, nwf_num,
                                              dst_offs, dst_weight,
-                                             Rsum_factor, nword_factor);
+                                             nword_factor);
     }
     CrdTo->url_id= CrdFrom->url_id;
     CrdTo++;
@@ -1954,8 +2109,9 @@ int UdmParseQueryString(UDM_AGENT * Agent,
 
 
 
-static int
-UdmWordInWWList(UDM_WIDEWORDLIST *List, int *tok, size_t flen, int hlstop)
+static UDM_WIDEWORD*
+UdmWordInWWList(UDM_WIDEWORDLIST *List, int *tok, size_t flen,
+                int hlstop, size_t phrpos)
 {
   size_t uw;
   
@@ -1963,23 +2119,25 @@ UdmWordInWWList(UDM_WIDEWORDLIST *List, int *tok, size_t flen, int hlstop)
   {
     size_t slen;
     UDM_WIDEWORD *W= &List->Word[uw];
+    if (W->phrpos != phrpos)
+      continue;
     if (!hlstop && W->origin == UDM_WORD_ORIGIN_STOP)
       continue;
     slen= W->ulen;
     if (flen < slen)
       continue;
     if (flen == slen && !UdmUniStrNCaseCmp(tok, W->uword, slen))
-      return 1;
+      return W;
       
     if (flen > slen) switch (List->wm)
     {
       case UDM_MATCH_BEGIN:
         if (!UdmUniStrNCaseCmp(tok, W->uword, slen))
-          return 1;
+          return W;
         break;
       case UDM_MATCH_END:
         if (!UdmUniStrNCaseCmp(tok + flen - slen, W->uword, slen))
-          return 1;
+          return W;
         break;
       case UDM_MATCH_SUBSTR:
         {
@@ -1994,7 +2152,7 @@ UdmWordInWWList(UDM_WIDEWORDLIST *List, int *tok, size_t flen, int hlstop)
             }
             if (l2 == slen)
             {
-              return 1;
+              return W;
               break;
             }
           }
@@ -2002,10 +2160,74 @@ UdmWordInWWList(UDM_WIDEWORDLIST *List, int *tok, size_t flen, int hlstop)
         break;
     }
   }
-  return 0;
+  return NULL;
 }
 
 
+/*
+  Remove hilight markers from a string and return
+  its new length, in bytes.
+*/
+static size_t
+UdmRemoveHl(UDM_CHARSET *cs, char *str, size_t from, size_t to)
+{
+  if (cs == &udm_charset_sys_int)
+  {
+    int *stri= (int*) str;
+    int *s= stri + from / sizeof(int);
+    int *e= stri + to / sizeof(int);
+    int *dst= s;
+    for (; s < e; s++)
+    { 
+      if (*s != 2 && *s != 3)
+        *dst++= *s;
+    }
+    return (dst - stri) * sizeof(int);
+  }
+  else
+  {
+    char *s= str + from, *e= str + to, *dst= s;
+    for (; s < e; s++)
+    { 
+      if (*s != 2 && *s != 3)
+        *dst++= *s;
+    }
+    return dst - str;
+  }
+}
+
+
+static size_t
+UdmHlAppend(UDM_CONV *uni_bc, UDM_WIDEWORD *found,
+            char *dst, size_t dstlen, size_t dstmaxlen,
+            int *tok, size_t toklen)
+{
+  int i2= 2, i3= 3;
+
+  if (found)
+  {
+    dstlen+= UdmConv(uni_bc, dst + dstlen, dstmaxlen, (char*) &i2, sizeof(i2));
+  }
+  if (uni_bc->to == &udm_charset_sys_int)
+  {
+    memcpy(dst + dstlen, tok, sizeof(*tok) * toklen);
+    dstlen+= sizeof(*tok) * toklen;
+  }
+  else
+    dstlen+= UdmConv(uni_bc, dst + dstlen, dstmaxlen, (char*) tok, sizeof(*tok) * toklen);
+
+  if (found)
+    dstlen+= UdmConv(uni_bc, dst + dstlen, dstmaxlen, (char*) &i3, sizeof(i3));
+
+  /*fprintf(stderr, "appended to '%.*s'\n", dstlen, dst);*/
+
+  return dstlen;
+}
+
+/*
+#define DEBUG_HL 0
+*/
+
 /* Returns a 0-terminated string */
 
 static char *
@@ -2014,12 +2236,16 @@ UdmHlConvertExtWithConv(UDM_WIDEWORDLIST *List, const char *src,
                         int hilight_stopwords)
 {
   int		*tok, *lt, ctype, *uni;
-  int           i0= 0, i2= 2, i3= 3;
+  int           i0= 0;
   char          *dst;
-  size_t        srclen, dstlen= 0, fndlen= 0;
-  size_t        dstmaxlen, unimaxlen;
+  size_t        srclen, dstlen= 0, dstlen_phr= 0;
+  size_t        dstmaxlen, unimaxlen, expected_phrpos= 0;
   UDM_CHARSET   *sys_int= &udm_charset_sys_int;
 
+#ifdef DEBUG_HL
+  fprintf(stderr, "tocs=%s src='%s'\n", uni_bc->to->name, src);
+#endif
+
   if(!src)return NULL;
   if ((srclen = strlen(src)) == 0) return NULL;
   
@@ -2035,28 +2261,49 @@ UdmHlConvertExtWithConv(UDM_WIDEWORDLIST *List, const char *src,
   for (tok= UdmUniGetSepToken(uni, &lt, &ctype) ; tok ;
        tok= UdmUniGetSepToken(NULL, &lt, &ctype))
   {
-    int found= 0;
-    size_t flen= lt - tok;
+    size_t toklen= lt - tok;
 
-    /* Check that it is word to be marked */
-    if (ctype != UDM_UNI_SEPAR && List != NULL)
-      found= UdmWordInWWList(List, tok, flen, hilight_stopwords);
-    
-    if (found)
+    if (ctype == UDM_UNI_SEPAR || !List)
     {
-      dstlen+= UdmConv(uni_bc, dst + dstlen, dstmaxlen, (char*) &i2, sizeof(i2));
-      fndlen+= flen;
-    }
-    if (uni_bc->to == sys_int)
-    {
-      memcpy(dst + dstlen, tok, sizeof(*tok) * flen);
-      dstlen+= sizeof(*tok) * flen;
+      dstlen= UdmHlAppend(uni_bc, NULL, dst, dstlen, dstmaxlen, tok, toklen);
     }
     else
-      dstlen+= UdmConv(uni_bc, dst + dstlen, dstmaxlen, (char*) tok, sizeof(*tok) * flen);
-    
-    if (found)
-      dstlen+= UdmConv(uni_bc, dst + dstlen, dstmaxlen, (char*) &i3, sizeof(i3));
+    {
+      UDM_WIDEWORD *found;
+      found= UdmWordInWWList(List, tok, toklen, hilight_stopwords, expected_phrpos);
+      dstlen= UdmHlAppend(uni_bc, found, dst, dstlen, dstmaxlen, tok, toklen);
+      if (found)
+      {
+        if (found->phrpos + 1 == found->phrlen)
+        {
+          /* last in phrase found */
+          expected_phrpos= 0;
+          dstlen_phr= dstlen;
+        }
+        else
+        {
+          /* middle in phrase found */
+          expected_phrpos++;
+        }
+      }
+      else
+      {
+        /* No word found on expected phrase position, rollback */
+        dstlen= UdmRemoveHl(uni_bc->to, dst, dstlen_phr, dstlen);
+        dstlen_phr= dstlen;
+        expected_phrpos= 0;
+      }
+    }
+  }
+
+
+#ifdef DEBUG_HL  
+  fprintf(stderr, "end: expected_phrpos=%d dstlen=%d dstlen_phr=%d\n", expected_phrpos, dstlen, dstlen_phr);
+#endif
+  if (expected_phrpos > 0)
+  {
+    /* Roll back: incomplete last phrase */
+    dstlen= UdmRemoveHl(uni_bc->to, dst, dstlen_phr, dstlen);
   }
 
   dstlen+= UdmConv(uni_bc, dst + dstlen, dstmaxlen, (char*) &i0, sizeof(i0));
