@@ -77,6 +77,12 @@ static int UdmMySQLInit(UDM_DB *db)
     return UDM_ERROR;
   }
   db->connected=1;
+#if NOT_YET
+  /* mysql_get_server_version appeared in 4.1.x */
+  db->version= mysql_get_server_version(mysql);
+  if (db->version >= 40100)
+    db->flags|= UDM_SQL_HAVE_SUBSELECT;
+#endif
   if (DBLog)
   {
     char qbuf[64];
