@@ -95,7 +95,40 @@ static size_t PgUnescape(char *dst, const char *src, size_t length)
 }
 
 
-static int UdmPgSQLQuery(UDM_DB *db, UDM_SQLRES *res, const char *q)
+
+static int UdmPgSQLResultUnescape(UDM_SQLRES *res)
+{
+  size_t row;
+  res->Items= (UDM_PSTR*)UdmMalloc(res->nRows * res->nCols * sizeof(UDM_PSTR));
+  
+  for (row= 0; row < res->nRows; row++)
+  {
+    size_t col, offs= res->nCols * row;
+    for(col= 0; col < res->nCols; col++)
+    {
+      UDM_PSTR *I= &res->Items[offs + col];
+      const char *ptr= PQgetvalue(res->pgsqlres,(int) row, (int) col);
+      size_t len= PQgetlength(res->pgsqlres, (int) row, (int )col);
+      I->val= (char*) UdmMalloc(len + 1);
+      if (PQftype(res->pgsqlres, (int) col) == 17)
+      {
+        I->len= PgUnescape(I->val, ptr, len);
+        I->val[I->len]= '\0';
+      }
+      else
+      {
+        memcpy(I->val, ptr, len);
+        I->val[len]= '\0';
+        I->len= len;
+      }
+    }
+  }
+  return UDM_OK;
+}
+
+
+static int UdmPgSQLQueryInternal(UDM_DB *db, UDM_SQLRES *res,
+                                 const char *q, int unescape)
 {
   size_t i;
   PGresult *PGres;
@@ -172,36 +205,24 @@ static int UdmPgSQLQuery(UDM_DB *db, UDM_SQLRES *res, const char *q)
     if (PQftype(res->pgsqlres, (int) i) == 17)
       have_bytea++;
   }
-  if (have_bytea)
-  {
-    size_t row;
-    res->Items= (UDM_PSTR*)UdmMalloc(res->nRows * res->nCols * sizeof(UDM_PSTR));
-    for (row= 0; row < res->nRows; row++)
-    {
-      size_t col, offs= res->nCols * row;
-      for(col= 0; col < res->nCols; col++)
-      {
-        UDM_PSTR *I= &res->Items[offs + col];
-        const char *ptr= PQgetvalue(res->pgsqlres,(int) row, (int) col);
-        size_t len= PQgetlength(res->pgsqlres, (int) row, (int )col);
-        I->val= (char*) UdmMalloc(len + 1);
-        if (PQftype(res->pgsqlres, (int) col) == 17)
-        {
-          I->len= PgUnescape(I->val, ptr, len);
-          I->val[I->len]= '\0';
-        }
-        else
-        {
-          memcpy(I->val, ptr, len);
-          I->val[len]= '\0';
-          I->len= len;
-        }
-      }
-    }
-  }
+
+  if (have_bytea && unescape)
+    UdmPgSQLResultUnescape(res);
+
   return UDM_OK;
 }
 
+static int UdmPgSQLQuery(UDM_DB *db, UDM_SQLRES *res, const char *q)
+{
+  return UdmPgSQLQueryInternal(db, res, q, 1);
+}
+
+static int UdmPgSQLExecDirect(UDM_DB *db, UDM_SQLRES *res, const char *q)
+{
+  return UdmPgSQLQueryInternal(db, res, q, 0);
+}
+
+
 static void UdmPgSQLClose(UDM_DB *db)
 {
   PQfinish((PGconn*)db->specific);
@@ -256,7 +277,7 @@ UDM_SQLDB_HANDLER udm_sqldb_pgsql_handler =
   UdmPgSQLFetchRow,
   UdmSQLStoreResultSimple,
   UdmSQLFreeResultSimple,
-  UdmPgSQLQuery
+  UdmPgSQLExecDirect
 };
 
 #endif
