@@ -86,6 +86,10 @@
 #include <sqlite.h>
 #endif
 
+#if HAVE_SQLITE3
+#include <sqlite3.h>
+#endif
+
 #ifdef WIN32
 #include <process.h>
 #endif
@@ -1438,6 +1442,7 @@ UDM_SQLDB_HANDLER udm_sqldb_ctlib_handler =
 
 
 /*********************************** SQLITE *********************/
+
 #if (HAVE_SQLITE)
 
 static int UdmSQLiteInitDB(UDM_DB *db)
@@ -1575,6 +1580,152 @@ UDM_SQLDB_HANDLER udm_sqldb_sqlite_handler =
 #endif  /* HAVE_SQLITE */
 
 
+#if (HAVE_SQLITE3)
+
+#include <sqlite3.h>
+
+static int UdmSQLite3InitDB(UDM_DB *db)
+{
+  sqlite3 *ppDb;
+  char edbname[1024];
+  char dbname[1024], *e;
+  strncpy(edbname,db->DBName,sizeof(dbname));
+  dbname[sizeof(edbname)-1]='\0';
+  
+  UdmUnescapeCGIQuery(dbname,edbname);
+  /* Remove possible trailing slash */
+  e= dbname+strlen(dbname);
+  if (e>dbname && e[-1]=='/')
+    e[-1]='\0';
+  
+  if (SQLITE_OK != sqlite3_open(dbname, &ppDb))
+  {
+    const char *errmsg= sqlite3_errmsg(ppDb);
+    udm_snprintf(db->errstr, sizeof(db->errstr),
+                "sqlite3 driver: %s", errmsg ? errmsg : "<NOERROR>");
+    db->errcode= 1;
+    sqlite3_close(ppDb);
+    return UDM_ERROR;
+  }
+  db->specific= (void*) ppDb;
+  db->connected= 1;
+  sqlite3_busy_timeout((sqlite3*)(db->specific),30*1000);
+  return UDM_OK;
+}
+
+static int SQLite3CallBack(void *pArg, int argc, char **argv, char **name)
+{
+  UDM_SQLRES *res= (UDM_SQLRES*) pArg;
+  int i;
+
+  if (res->nCols == 0)
+  {
+    res->nCols= argc;
+    for (i=0 ; i<argc; i++)
+    {
+      res->Fields=(UDM_SQLFIELD*)UdmMalloc(res->nCols*sizeof(UDM_SQLFIELD));
+      bzero(res->Fields,res->nCols*sizeof(UDM_SQLFIELD));
+      for (i=0 ; i < res->nCols; i++)
+      {
+        res->Fields[i].sqlname = (char*)UdmStrdup(name[i]);
+        res->Fields[i].sqllen= 0;
+        res->Fields[i].sqltype= 0;
+      }
+    }
+  }
+  res->nRows++;
+  res->Items=(UDM_PSTR*)UdmRealloc(res->Items,res->nRows*res->nCols*sizeof(UDM_PSTR));
+  
+  for (i = 0; i < argc; i++)
+  {
+    size_t offs= (res->nRows-1)*res->nCols+(size_t)i;
+    size_t len= strlen(argv[i]?argv[i]:"");
+    res->Items[offs].len= len;
+    res->Items[offs].val= (char*)UdmMalloc(len+1);
+    memcpy(res->Items[offs].val,argv[i]?argv[i]:"",len+1);
+  } 
+  return 0;
+}
+
+
+static int UdmSQLite3Query(UDM_DB *db, UDM_SQLRES *res, const char *q)
+{
+  char *errmsg;
+  int rc;
+  
+  if (res)
+  {
+    bzero((void*) res, sizeof(UDM_SQLRES));
+    res->db= db;
+  }
+  
+#ifdef WIN32
+  /** Remove heading "/" from DBName (in case of absolute path) */
+  if(db->DBName[2] == ':')
+  {
+    char *tmp_dbname = UdmStrdup(db->DBName + 1);
+    UDM_FREE(db->DBName);
+    db->DBName = tmp_dbname;
+  }
+#endif /* #ifdef WIN32 */
+
+  db->errcode= 0;
+  db->errstr[0]='\0';
+  if(!db->connected && UDM_OK != UdmSQLite3InitDB(db))
+    return UDM_ERROR;
+  
+  if ((rc= sqlite3_exec((sqlite3 *)db->specific, q, SQLite3CallBack, (void*)res, &errmsg)))
+  {
+    udm_snprintf(db->errstr, sizeof(db->errstr),
+                 "sqlite3 driver: %s",errmsg ? errmsg : "<NOERROR>");
+    sqlite3_free(errmsg);
+    if (!strstr(db->errstr,"unique"))
+    {
+      db->errcode=1;
+      return UDM_ERROR;
+    }
+  }
+  return UDM_OK;
+}
+
+static void UdmSQLite3Close(UDM_DB *db)
+{
+  sqlite3_close((sqlite3*)(db->specific));
+  db->specific = NULL;
+}
+
+static int UdmSQLite3Begin(UDM_DB *db)
+{
+  return UdmSQLite3Query(db,NULL,"BEGIN TRANSACTION");
+}
+
+static int UdmSQLite3Commit(UDM_DB *db)
+{
+  return UdmSQLite3Query(db,NULL,"COMMIT");
+}
+
+
+UDM_SQLDB_HANDLER udm_sqldb_sqlite3_handler =
+{
+  NULL,
+  UdmSQLite3Query,
+  UdmSQLite3Close,
+  UdmSQLite3Begin,
+  UdmSQLite3Commit,
+  NULL,
+  NULL,
+  NULL,
+  UdmSQLFetchRowSimple,
+  UdmSQLStoreResultSimple,
+  UdmSQLFreeResultSimple,
+  UdmSQLite3Query
+};
+
+#endif  /* HAVE_SQLITE3 */
+
+
+
+
 /***********************************************************************/
 
 /*
@@ -1597,7 +1748,8 @@ char * UdmDBEscStr(int DBType,char *to,const char *from,size_t len)
       DBType == UDM_DB_MSSQL || DBType == UDM_DB_DB2 ||
       DBType == UDM_DB_IBASE || DBType == UDM_DB_SAPDB ||
       DBType == UDM_DB_SQLITE || DBType == UDM_DB_ACCESS ||
-      DBType == UDM_DB_MIMER || DBType == UDM_DB_CACHE)
+      DBType == UDM_DB_MIMER || DBType == UDM_DB_CACHE ||
+      DBType == UDM_DB_SQLITE3)
    {
      while(*from)
      {
@@ -1747,7 +1899,10 @@ int __UDMCALL UdmSQLFreeResultSimple(UDM_DB *db, UDM_SQLRES *res)
   if(res->db->DBDriver==UDM_DB_PGSQL)
   {
     PQclear(res->pgsqlres);
-    return UDM_OK;
+    /*
+      Continue to udb_free_results() to
+      free allocated memory if we have bytea datatype
+    */
   }
 #endif
 
