@@ -621,7 +621,7 @@ static int TemplateIncOrDec(UDM_TEMPLATE *tmpl)
 
 static int CreateArg(UDM_TEMPLATE *tmpl, const char *name, const char *arg)
 {
-  size_t vallen = 256 + 4 * strlen(arg);
+  size_t vallen = 4 * 256 + 4 * strlen(arg);
   char  *val = (char*)UdmMalloc(vallen);
   val[0]= '\0';
   PrintTextTemplate(tmpl->Agent, NULL, val, vallen, tmpl->vars,
@@ -1119,7 +1119,7 @@ static int HtmlTemplatePrgAddInvokeMethod(UDM_TMPL_PRG *prg,
   UDM_VARLIST attr;
   UDM_TMPL_PRGITEM i;
   const char *hlast= NULL;
-  const char *args[3];
+  const char *args[4];
   
   UdmHTMLTOKInit(&tag);
   UdmHTMLToken(template, &hlast, &tag);
