@@ -770,6 +770,7 @@ static UDM_CHARSET_ALIAS alias[]={
 	{"ansi_x3.4-1968",	UDM_CHARSET_US_ASCII},
 	{"arabic",		UDM_CHARSET_8859_6},
 	{"armscii-8",		UDM_CHARSET_ARMSCII_8},
+	{"armscii8",		UDM_CHARSET_ARMSCII_8},
 	{"ascii",		UDM_CHARSET_US_ASCII},
 	{"asmo-708",		UDM_CHARSET_8859_6},
 	{"big-5",		UDM_CHARSET_BIG5},
@@ -950,6 +951,8 @@ static UDM_CHARSET_ALIAS alias[]={
 	{"iso_8859-9:1989",	UDM_CHARSET_8859_9},
 	{"koi8-r",		UDM_CHARSET_KOI8_R},
 	{"koi8-u",		UDM_CHARSET_KOI8_U},
+	{"koi8r",		UDM_CHARSET_KOI8_R},
+	{"koi8u",		UDM_CHARSET_KOI8_U},
 	{"l1",			UDM_CHARSET_8859_1},
 	{"l2",			UDM_CHARSET_8859_2},
 	{"l3",			UDM_CHARSET_8859_3},
@@ -998,6 +1001,7 @@ static UDM_CHARSET_ALIAS alias[]={
 	{"tis-620",		UDM_CHARSET_8859_11},
 	{"tis620",		UDM_CHARSET_8859_11},
 	{"tscii",               UDM_CHARSET_TSCII},
+	{"ujis",		UDM_CHARSET_EUC_JP},
 	{"us",			UDM_CHARSET_US_ASCII},
 	{"us-ascii",		UDM_CHARSET_US_ASCII},
 	{"utf-8",		UDM_CHARSET_UTF8},
