@@ -928,7 +928,7 @@ time_t UdmHttpDate2Time_t(const char *date){
       */
       ds.tm_year= ((date[0] - '0') * 10 + (date[1] - '0') - 19) * 100 +
                   (date[2] - '0') * 10 + date[3] - '0';
-      ds.tm_mon= (date[5] - '0') * 10 + (date[6] - '0');
+      ds.tm_mon= (date[5] - '0') * 10 + (date[6] - '0') - 1;
       ds.tm_mday= (date[8] - '0') * 10 + (date[9] - '0');
       ds.tm_hour= 0;
       ds.tm_min= 0;
@@ -945,7 +945,7 @@ time_t UdmHttpDate2Time_t(const char *date){
       */
       ds.tm_year= ((date[6] - '0') * 10 + (date[7] - '0') - 19) * 100 +
                   (date[8] - '0') * 10 + date[9] - '0';
-      ds.tm_mon= (date[3] - '0') * 10 + (date[4] - '0');
+      ds.tm_mon= (date[3] - '0') * 10 + (date[4] - '0') - 1;
       ds.tm_mday= (date[0] - '0') * 10 + (date[1] - '0');
       ds.tm_hour= 0;
       ds.tm_min= 0;
@@ -1062,7 +1062,16 @@ time_t UdmHttpDate2Time_t(const char *date){
     ds.tm_mon = mon;
 
 ret:
-    /* printf("parseHTTPdate: %s is %lu\n", date, ap_tm2sec(&ds)); */
+
+#if 0
+    {
+      int res= ap_tm2sec(&ds);
+      char buf[128];
+      strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", &ds);
+      printf("parseHTTPdate: '%s' is %lu: strftime=%s, month=%d\n", date, res, buf, ds.tm_mon);
+      
+    }
+#endif
 
     return ap_tm2sec(&ds);
 }
@@ -2250,25 +2259,36 @@ static char udm_soundex_code(UDM_CHARSET *cset, const char *ptr)
 
 void UdmSoundex(UDM_CHARSET *cs, char *dst, const char *src, size_t srclen)
 {
-  char ch, prev, *end;
+  char ch, prev, *end, *padend, *dst0= dst;
   const char *srcend= src + srclen;
 
+  while (!isalpha((unsigned char)src[0]) && srclen > 0)
+  {
+    src++;
+    srclen--;
+  }
+
   *dst++= toupper((unsigned char)*src);
   prev= udm_soundex_code(cs, src);
-  end= dst + 3;
+  end= dst + 24;
+  padend= dst + 3;
 
   src++;
-  while (dst < end && (ch= udm_soundex_code(cs, src)) != 0 && src < srcend)
+  for ( ; dst < end && src < srcend; src++)
   {
-    src++;
+    ch= udm_soundex_code(cs, src);
+    if (!isalpha((unsigned char)src[0]))
+      continue;
     if ((ch != '0') && (ch != prev)) /* if not skipped or double */
     {
       *dst++ = ch;
+      prev = ch;
     }
-    prev = ch;
   }
-  while (dst < end)
-    *dst++ = '0';
+
+  while (dst < padend)
+   *dst++ = '0';
+
   *dst=0;
 }
 
